"""Test cases for the ISO20022 validation system."""
from decimal import Decimal
from datetime import datetime
from xsdata.models.datatype import XmlDateTime
from pyiso20022.pacs.pacs_008_001_08.pacs_008_001_08 import (
    PaymentIdentification7,
    ActiveCurrencyAndAmount,
    Document,
    FitoFicustomerCreditTransferV08,
    GroupHeader93,
    CreditTransferTransaction39,
    BranchAndFinancialInstitutionIdentification6,
    FinancialInstitutionIdentification18,
    PartyIdentification135,
    SettlementInstruction7,
    SettlementMethod1Code,
    ChargeBearerType1Code,
)
from pyiso20022.tools.validation import (
    validate_instance,
    validate_message,
    validate_field,
    get_constraints,
    ValidationError,
    ValidationResult,
)


def test_validate_uetr_pattern_valid():
    """Test UETR field validation with valid UUID pattern."""
    # Valid UETR (UUID v4 format)
    valid_uetr = "550e8400-e29b-41d4-a716-446655440000"

    result = validate_field(
        PaymentIdentification7,
        'uetr',
        valid_uetr
    )

    assert result.is_valid
    assert len(result.errors) == 0


def test_validate_uetr_pattern_invalid():
    """Test UETR field validation with invalid pattern."""
    # Invalid UETR (wrong format)
    invalid_uetr = "not-a-valid-uuid"

    result = validate_field(
        PaymentIdentification7,
        'uetr',
        invalid_uetr
    )

    assert not result.is_valid
    assert len(result.errors) == 1
    assert result.errors[0].constraint == 'pattern'
    assert 'does not match required pattern' in result.errors[0].message


def test_validate_string_length_constraints():
    """Test string length validation."""
    # Test min_length constraint
    result = validate_field(
        PaymentIdentification7,
        'end_to_end_id',
        ""  # Empty string, min_length is 1
    )

    assert not result.is_valid
    assert result.errors[0].constraint == 'min_length'

    # Test max_length constraint
    long_string = "a" * 50  # max_length is 35
    result = validate_field(
        PaymentIdentification7,
        'end_to_end_id',
        long_string
    )

    assert not result.is_valid
    assert result.errors[0].constraint == 'max_length'


def test_validate_decimal_constraints():
    """Test decimal value validation."""
    # Test valid amount
    valid_amount = ActiveCurrencyAndAmount(
        value=Decimal('100.50'),
        ccy='USD'
    )

    result = validate_instance(valid_amount)
    assert result.is_valid

    # Test negative amount (min_inclusive is 0)
    invalid_amount = ActiveCurrencyAndAmount(
        value=Decimal('-10.00'),
        ccy='USD'
    )

    result = validate_instance(invalid_amount)
    assert not result.is_valid
    assert result.errors[0].constraint == 'min_inclusive'


def test_validate_currency_pattern():
    """Test currency code pattern validation."""
    # Valid currency code
    valid_amount = ActiveCurrencyAndAmount(
        value=Decimal('100.00'),
        ccy='USD'
    )

    result = validate_instance(valid_amount)
    assert result.is_valid

    # Invalid currency code (not 3 uppercase letters)
    invalid_amount = ActiveCurrencyAndAmount(
        value=Decimal('100.00'),
        ccy='us'  # Should be 3 uppercase letters
    )

    result = validate_instance(invalid_amount)
    assert not result.is_valid
    assert result.errors[0].constraint == 'pattern'


def test_validate_enum_values():
    """Test enum value validation."""
    # This would test enum validation if we had an instance that uses enums
    # For now, we can test that the enum type is recognized
    constraints = get_constraints(PaymentIdentification7)

    # Verify that constraints are extracted properly
    assert 'uetr' in constraints
    assert 'pattern' in constraints['uetr']


def test_validate_required_fields():
    """Test required field validation."""
    # Test with missing required field
    payment_id = PaymentIdentification7(
        end_to_end_id=None  # This is required
    )

    result = validate_instance(payment_id)
    assert not result.is_valid
    assert result.errors[0].constraint == 'required'


def test_validate_complete_instance():
    """Test validation of a complete valid instance."""
    payment_id = PaymentIdentification7(
        instr_id="INSTR123",
        end_to_end_id="E2E123",
        tx_id="TX123",
        uetr="550e8400-e29b-41d4-a716-446655440000"
    )

    result = validate_instance(payment_id)
    assert result.is_valid
    assert len(result.errors) == 0


def test_get_constraints():
    """Test constraint extraction functionality."""
    constraints = get_constraints(PaymentIdentification7)

    # Check that constraints are properly extracted
    assert 'uetr' in constraints
    assert 'end_to_end_id' in constraints

    # Check UETR constraints
    uetr_constraints = constraints['uetr']
    assert 'pattern' in uetr_constraints
    expected_pattern = (
        r"[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-"
        r"[89ab][a-f0-9]{3}-[a-f0-9]{12}"
    )
    assert uetr_constraints['pattern'] == expected_pattern

    # Check end_to_end_id constraints
    e2e_constraints = constraints['end_to_end_id']
    assert 'required' in e2e_constraints
    assert 'min_length' in e2e_constraints
    assert 'max_length' in e2e_constraints
    assert e2e_constraints['required'] is True
    assert e2e_constraints['min_length'] == 1
    assert e2e_constraints['max_length'] == 35


def test_validation_result_string_representation():
    """Test ValidationResult string representation."""
    # Test valid result
    result = ValidationResult()
    assert str(result) == "Validation passed"

    # Test invalid result
    result.add_error('test_field', 'test_value', 'test_constraint',
                     'Test error message')
    result_str = str(result)
    assert "Validation failed with 1 error(s)" in result_str
    assert "Test error message" in result_str

    # Test with warnings
    result.add_warning("Test warning")
    result_str = str(result)
    assert "Warnings (1)" in result_str
    assert "Test warning" in result_str


def test_validation_error_properties():
    """Test ValidationError properties."""
    error = ValidationError(
        'test_field', 'test_value', 'test_constraint', 'Test message'
    )

    assert error.field_name == 'test_field'
    assert error.value == 'test_value'
    assert error.constraint == 'test_constraint'
    assert error.message == 'Test message'
    assert str(error) == "Field 'test_field': Test message"


def test_strict_vs_non_strict_validation():
    """Test difference between strict and non-strict validation."""
    # Create an instance with multiple validation errors
    payment_id = PaymentIdentification7(
        end_to_end_id="",  # Too short (min_length = 1)
        tx_id="a" * 50,    # Too long (max_length = 35)
        uetr="invalid-uuid"  # Invalid pattern
    )

    # Strict validation (stops at first error)
    strict_result = validate_instance(payment_id, strict=True)
    assert not strict_result.is_valid
    assert len(strict_result.errors) == 1

    # Non-strict validation (collects all errors)
    non_strict_result = validate_instance(payment_id, strict=False)
    assert not non_strict_result.is_valid
    assert len(non_strict_result.errors) > 1


def test_validate_complete_pacs008_message():
    """Test validation of a complete PACS.008 message."""
    # Create a complete PACS.008 message
    settlement_instruction = SettlementInstruction7(
        sttlm_mtd=SettlementMethod1Code.CLRG
    )

    group_header = GroupHeader93(
        msg_id="MSG123456789",
        cre_dt_tm=XmlDateTime.from_datetime(datetime.now()),
        nb_of_txs="1",
        sttlm_inf=settlement_instruction
    )

    # Create financial institution
    fin_inst = FinancialInstitutionIdentification18(
        bicfi="BANKGB2L"
    )

    dbtr_agt = BranchAndFinancialInstitutionIdentification6(
        fin_instn_id=fin_inst
    )

    cdtr_agt = BranchAndFinancialInstitutionIdentification6(
        fin_instn_id=fin_inst
    )

    # Create parties
    debtor = PartyIdentification135(
        nm="John Doe"
    )

    creditor = PartyIdentification135(
        nm="Jane Smith"
    )

    # Create payment identification
    payment_id = PaymentIdentification7(
        end_to_end_id="E2E123456789"
    )

    # Create settlement amount
    settlement_amount = ActiveCurrencyAndAmount(
        value=Decimal('1000.00'),
        ccy='USD'
    )

    # Create credit transfer transaction
    credit_transfer = CreditTransferTransaction39(
        pmt_id=payment_id,
        intr_bk_sttlm_amt=settlement_amount,
        chrg_br=ChargeBearerType1Code.SHAR,
        dbtr=debtor,
        dbtr_agt=dbtr_agt,
        cdtr=creditor,
        cdtr_agt=cdtr_agt
    )

    # Create the main message
    fi_to_fi_transfer = FitoFicustomerCreditTransferV08(
        grp_hdr=group_header,
        cdt_trf_tx_inf=[credit_transfer]
    )

    # Create the document
    document = Document(
        fito_ficstmr_cdt_trf=fi_to_fi_transfer
    )

    # Validate the complete message
    result = validate_message(document)
    assert result.is_valid, f"Validation failed: {result}"
    assert len(result.errors) == 0


def test_validate_pacs008_message_with_errors():
    """Test validation of PACS.008 message with validation errors."""
    # Create a PACS.008 message with validation errors
    settlement_instruction = SettlementInstruction7(
        sttlm_mtd=SettlementMethod1Code.CLRG
    )

    group_header = GroupHeader93(
        msg_id="",  # Invalid: empty string (min_length = 1)
        cre_dt_tm=XmlDateTime.from_datetime(datetime.now()),
        nb_of_txs="1",
        sttlm_inf=settlement_instruction
    )

    # Create financial institution with invalid BIC
    fin_inst = FinancialInstitutionIdentification18(
        bicfi="INVALID"  # Invalid BIC format
    )

    dbtr_agt = BranchAndFinancialInstitutionIdentification6(
        fin_instn_id=fin_inst
    )

    cdtr_agt = BranchAndFinancialInstitutionIdentification6(
        fin_instn_id=fin_inst
    )

    # Create parties
    debtor = PartyIdentification135(
        nm="John Doe"
    )

    creditor = PartyIdentification135(
        nm="Jane Smith"
    )

    # Create payment identification with invalid UETR
    payment_id = PaymentIdentification7(
        end_to_end_id="E2E123456789",
        uetr="invalid-uetr-format"  # Invalid UETR pattern
    )

    # Create settlement amount with invalid currency
    settlement_amount = ActiveCurrencyAndAmount(
        value=Decimal('-100.00'),  # Invalid: negative amount
        ccy='INVALID'  # Invalid currency code
    )

    # Create credit transfer transaction
    credit_transfer = CreditTransferTransaction39(
        pmt_id=payment_id,
        intr_bk_sttlm_amt=settlement_amount,
        chrg_br=ChargeBearerType1Code.SHAR,
        dbtr=debtor,
        dbtr_agt=dbtr_agt,
        cdtr=creditor,
        cdtr_agt=cdtr_agt
    )

    # Create the main message
    fi_to_fi_transfer = FitoFicustomerCreditTransferV08(
        grp_hdr=group_header,
        cdt_trf_tx_inf=[credit_transfer]
    )

    # Create the document
    document = Document(
        fito_ficstmr_cdt_trf=fi_to_fi_transfer
    )

    # Validate the complete message
    result = validate_message(document, strict=False)
    assert not result.is_valid
    assert len(result.errors) > 0

    # Check that field paths are properly constructed
    error_fields = [error.field_name for error in result.errors]

    # Should have nested field paths like:
    # fito_ficstmr_cdt_trf.grp_hdr.msg_id
    # fito_ficstmr_cdt_trf.cdt_trf_tx_inf[0].pmt_id.uetr
    # etc.
    nested_errors = [field for field in error_fields if '.' in field]
    assert len(nested_errors) > 0, "Should have nested field path errors"


def test_validate_message_with_list_items():
    """Test validation of message with list items containing errors."""
    # Create multiple credit transfer transactions with errors
    payment_id1 = PaymentIdentification7(
        end_to_end_id=""  # Invalid: empty string
    )

    payment_id2 = PaymentIdentification7(
        end_to_end_id="E2E123",
        uetr="invalid-uetr"  # Invalid UETR
    )

    settlement_amount = ActiveCurrencyAndAmount(
        value=Decimal('100.00'),
        ccy='USD'
    )

    # Create financial institution
    fin_inst = FinancialInstitutionIdentification18(
        bicfi="BANKGB2L"
    )

    dbtr_agt = BranchAndFinancialInstitutionIdentification6(
        fin_instn_id=fin_inst
    )

    cdtr_agt = BranchAndFinancialInstitutionIdentification6(
        fin_instn_id=fin_inst
    )

    debtor = PartyIdentification135(nm="John Doe")
    creditor = PartyIdentification135(nm="Jane Smith")

    # Create transactions with errors
    credit_transfer1 = CreditTransferTransaction39(
        pmt_id=payment_id1,  # Has validation error
        intr_bk_sttlm_amt=settlement_amount,
        chrg_br=ChargeBearerType1Code.SHAR,
        dbtr=debtor,
        dbtr_agt=dbtr_agt,
        cdtr=creditor,
        cdtr_agt=cdtr_agt
    )

    credit_transfer2 = CreditTransferTransaction39(
        pmt_id=payment_id2,  # Has validation error
        intr_bk_sttlm_amt=settlement_amount,
        chrg_br=ChargeBearerType1Code.SHAR,
        dbtr=debtor,
        dbtr_agt=dbtr_agt,
        cdtr=creditor,
        cdtr_agt=cdtr_agt
    )

    # Validate the list directly
    transactions = [credit_transfer1, credit_transfer2]
    result = validate_message(transactions, strict=False)

    assert not result.is_valid
    assert len(result.errors) > 0

    # Check that list indices are included in field paths
    error_fields = [error.field_name for error in result.errors]
    list_errors = [field for field in error_fields if '[' in field]
    assert len(list_errors) > 0, ("Should have list index errors like "
                                  "[0].field_name")


def test_validate_list_field_string_constraints():
    """Test validation of list fields with string constraints on items."""
    from dataclasses import dataclass, field
    from typing import List
    
    @dataclass
    class TestAddressClass:
        """Test class with address lines having string constraints."""
        adr_line: List[str] = field(
            default_factory=list,
            metadata={
                "name": "AdrLine",
                "type": "Element",
                "max_occurs": 7,
                "min_length": 1,
                "max_length": 70,
            },
        )
    
    # Test 1: Valid address lines
    valid_address = TestAddressClass(
        adr_line=["123 Main Street", "Apt 4B", "New York, NY 10001"]
    )
    result = validate_instance(valid_address, strict=False)
    assert result.is_valid, f"Valid address lines should pass: {result}"
    
    # Test 2: Empty list is valid (no min_occurs)
    empty_address = TestAddressClass(adr_line=[])
    result = validate_instance(empty_address, strict=False)
    assert result.is_valid, "Empty list should be valid when no min_occurs"
    
    # Test 3: Too many items (exceeds max_occurs)
    too_many_lines = TestAddressClass(
        adr_line=["Line 1", "Line 2", "Line 3", "Line 4", 
                  "Line 5", "Line 6", "Line 7", "Line 8"]
    )
    result = validate_instance(too_many_lines, strict=False)
    assert not result.is_valid
    assert any(e.constraint == 'max_occurs' for e in result.errors)
    assert any('8 items but maximum allowed is 7' in e.message 
               for e in result.errors)
    
    # Test 4: Empty string in list (violates min_length)
    empty_string_address = TestAddressClass(
        adr_line=["123 Main Street", "", "New York, NY 10001"]
    )
    result = validate_instance(empty_string_address, strict=False)
    assert not result.is_valid
    assert any(e.constraint == 'min_length' for e in result.errors)
    assert any('adr_line[1]' in e.field_name for e in result.errors)
    
    # Test 5: String too long (exceeds max_length)
    long_string = "A" * 71  # 71 characters, exceeds max_length=70
    long_string_address = TestAddressClass(
        adr_line=["123 Main Street", long_string, "New York, NY 10001"]
    )
    result = validate_instance(long_string_address, strict=False)
    assert not result.is_valid
    assert any(e.constraint == 'max_length' for e in result.errors)
    assert any('adr_line[1]' in e.field_name for e in result.errors)
    assert any('71 exceeds maximum allowed length 70' in e.message 
               for e in result.errors)


def test_validate_list_field_with_min_occurs():
    """Test validation of list fields with min_occurs constraint."""
    from dataclasses import dataclass, field
    from typing import List
    
    @dataclass
    class TestRequiredListClass:
        """Test class with required list items."""
        required_items: List[str] = field(
            default_factory=list,
            metadata={
                "name": "RequiredItems",
                "type": "Element",
                "min_occurs": 2,
                "max_occurs": 5,
                "min_length": 1,
                "max_length": 50,
            },
        )
    
    # Test 1: Too few items (less than min_occurs)
    too_few = TestRequiredListClass(required_items=["Item 1"])
    result = validate_instance(too_few, strict=False)
    assert not result.is_valid
    assert any(e.constraint == 'min_occurs' for e in result.errors)
    assert any('1 items but minimum required is 2' in e.message 
               for e in result.errors)
    
    # Test 2: Valid number of items
    valid_list = TestRequiredListClass(
        required_items=["Item 1", "Item 2", "Item 3"]
    )
    result = validate_instance(valid_list, strict=False)
    assert result.is_valid
    
    # Test 3: Too many items (exceeds max_occurs)
    too_many = TestRequiredListClass(
        required_items=["Item 1", "Item 2", "Item 3", "Item 4", 
                        "Item 5", "Item 6"]
    )
    result = validate_instance(too_many, strict=False)
    assert not result.is_valid
    assert any(e.constraint == 'max_occurs' for e in result.errors)
    assert any('6 items but maximum allowed is 5' in e.message 
               for e in result.errors)


def test_validate_list_field_with_pattern():
    """Test validation of list fields with pattern constraints on items."""
    from dataclasses import dataclass, field
    from typing import List
    
    @dataclass
    class TestPatternListClass:
        """Test class with list items having pattern constraints."""
        codes: List[str] = field(
            default_factory=list,
            metadata={
                "name": "Codes",
                "type": "Element",
                "max_occurs": 10,
                "pattern": r"[A-Z]{3}[0-9]{3}",  # e.g., ABC123
            },
        )
    
    # Test 1: Valid codes
    valid_codes = TestPatternListClass(
        codes=["ABC123", "XYZ789", "DEF456"]
    )
    result = validate_instance(valid_codes, strict=False)
    assert result.is_valid
    
    # Test 2: Invalid pattern in one item
    invalid_pattern = TestPatternListClass(
        codes=["ABC123", "invalid", "XYZ789"]
    )
    result = validate_instance(invalid_pattern, strict=False)
    assert not result.is_valid
    assert any(e.constraint == 'pattern' for e in result.errors)
    assert any('codes[1]' in e.field_name for e in result.errors)
    assert any('does not match required pattern' in e.message 
               for e in result.errors)


def test_validate_postal_address_adr_line():
    """Test validation of PostalAddress24 adr_line field."""
    from pyiso20022.pacs.pacs_008_001_08.pacs_008_001_08 import PostalAddress24
    
    # Test 1: Valid address lines
    valid_address = PostalAddress24(
        adr_line=["123 Main Street", "Suite 100", "New York, NY 10001"],
        ctry="US"
    )
    result = validate_instance(valid_address, strict=False)
    assert result.is_valid
    
    # Test 2: Too many address lines (max_occurs=7)
    too_many_lines = PostalAddress24(
        adr_line=["Line 1", "Line 2", "Line 3", "Line 4", 
                  "Line 5", "Line 6", "Line 7", "Line 8"],
        ctry="US"
    )
    result = validate_instance(too_many_lines, strict=False)
    assert not result.is_valid
    assert any(e.constraint == 'max_occurs' for e in result.errors)
    
    # Test 3: Address line too long (max_length=70)
    long_line = "A" * 71
    long_address = PostalAddress24(
        adr_line=["123 Main Street", long_line],
        ctry="US"
    )
    result = validate_instance(long_address, strict=False)
    assert not result.is_valid
    assert any(e.constraint == 'max_length' for e in result.errors)
    assert any('adr_line[1]' in e.field_name for e in result.errors)



from pyiso20022.colr.colr_006_001_03.colr_006_001_03 import (
    CollateralAccount1,
    CollateralAccountIdentificationType1Choice,
    CollateralAccountType1Code,
    CollateralCancellationStatus1,
    CollateralManagementCancellationStatusV03,
    DateAndDateTimeChoice,
    Document,
    ExposureType5Code,
    GenericIdentification29,
    NameAndAddress6,
    Obligation3,
    PartyIdentification33Choice,
    PostalAddress2,
    Reference16,
    RejectionReasonV021Code,
    RejectionStatus2,
    Status4Code,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
)

__all__ = [
    "CollateralAccount1",
    "CollateralAccountIdentificationType1Choice",
    "CollateralAccountType1Code",
    "CollateralCancellationStatus1",
    "CollateralManagementCancellationStatusV03",
    "DateAndDateTimeChoice",
    "Document",
    "ExposureType5Code",
    "GenericIdentification29",
    "NameAndAddress6",
    "Obligation3",
    "PartyIdentification33Choice",
    "PostalAddress2",
    "Reference16",
    "RejectionReasonV021Code",
    "RejectionStatus2",
    "Status4Code",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
]



from pyiso20022.pain.pain_014_001_03.pain_014_001_03 import (
    AccountIdentification4Choice,
    AccountSchemeName1Choice,
    ActiveOrHistoricCurrencyAndAmount,
    AddressType2Code,
    AmountType3Choice,
    BranchAndFinancialInstitutionIdentification5,
    BranchData2,
    CashAccount24,
    CashAccountType2Choice,
    CategoryPurpose1Choice,
    Charges2,
    ClearingSystemIdentification2Choice,
    ClearingSystemMemberIdentification2,
    ContactDetails2,
    CreditDebitCode,
    CreditorPaymentActivationRequestStatusReportV03,
    CreditorReferenceInformation2,
    CreditorReferenceType1Choice,
    CreditorReferenceType2,
    DateAndPlaceOfBirth,
    Document,
    DocumentAdjustment1,
    DocumentType3Code,
    DocumentType5Code,
    EquivalentAmount2,
    FinancialIdentificationSchemeName1Choice,
    FinancialInstitutionIdentification8,
    GenericAccountIdentification1,
    GenericFinancialIdentification1,
    GenericOrganisationIdentification1,
    GenericPersonIdentification1,
    GroupHeader46,
    LocalInstrument2Choice,
    NamePrefix1Code,
    NumberOfTransactionsPerStatus3,
    OrganisationIdentification8,
    OrganisationIdentificationSchemeName1Choice,
    OriginalGroupInformation25,
    OriginalPaymentInstruction9,
    OriginalTransactionReference19,
    Party11Choice,
    PartyIdentification43,
    PaymentMethod4Code,
    PaymentTransaction49,
    PaymentTypeInformation19,
    PersonIdentification5,
    PersonIdentificationSchemeName1Choice,
    PostalAddress6,
    Priority2Code,
    ReferredDocumentInformation3,
    ReferredDocumentType1Choice,
    ReferredDocumentType2,
    RemittanceAmount1,
    RemittanceInformation6,
    ServiceLevel8Choice,
    StatusReason6Choice,
    StatusReasonInformation9,
    StructuredRemittanceInformation8,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
    TransactionGroupStatus3Code,
    TransactionIndividualStatus3Code,
)

__all__ = [
    "AccountIdentification4Choice",
    "AccountSchemeName1Choice",
    "ActiveOrHistoricCurrencyAndAmount",
    "AddressType2Code",
    "AmountType3Choice",
    "BranchAndFinancialInstitutionIdentification5",
    "BranchData2",
    "CashAccount24",
    "CashAccountType2Choice",
    "CategoryPurpose1Choice",
    "Charges2",
    "ClearingSystemIdentification2Choice",
    "ClearingSystemMemberIdentification2",
    "ContactDetails2",
    "CreditDebitCode",
    "CreditorPaymentActivationRequestStatusReportV03",
    "CreditorReferenceInformation2",
    "CreditorReferenceType1Choice",
    "CreditorReferenceType2",
    "DateAndPlaceOfBirth",
    "Document",
    "DocumentAdjustment1",
    "DocumentType3Code",
    "DocumentType5Code",
    "EquivalentAmount2",
    "FinancialIdentificationSchemeName1Choice",
    "FinancialInstitutionIdentification8",
    "GenericAccountIdentification1",
    "GenericFinancialIdentification1",
    "GenericOrganisationIdentification1",
    "GenericPersonIdentification1",
    "GroupHeader46",
    "LocalInstrument2Choice",
    "NamePrefix1Code",
    "NumberOfTransactionsPerStatus3",
    "OrganisationIdentification8",
    "OrganisationIdentificationSchemeName1Choice",
    "OriginalGroupInformation25",
    "OriginalPaymentInstruction9",
    "OriginalTransactionReference19",
    "Party11Choice",
    "PartyIdentification43",
    "PaymentMethod4Code",
    "PaymentTransaction49",
    "PaymentTypeInformation19",
    "PersonIdentification5",
    "PersonIdentificationSchemeName1Choice",
    "PostalAddress6",
    "Priority2Code",
    "ReferredDocumentInformation3",
    "ReferredDocumentType1Choice",
    "ReferredDocumentType2",
    "RemittanceAmount1",
    "RemittanceInformation6",
    "ServiceLevel8Choice",
    "StatusReason6Choice",
    "StatusReasonInformation9",
    "StructuredRemittanceInformation8",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
    "TransactionGroupStatus3Code",
    "TransactionIndividualStatus3Code",
]



from pyiso20022.colr.colr_010_001_03.colr_010_001_03 import (
    AccountIdentification4Choice,
    AccountIdentification26,
    AccountSchemeName1Choice,
    ActiveCurrencyAndAmount,
    ActiveOrHistoricCurrencyAnd13DecimalAmount,
    AddressType2Code,
    Agreement2,
    AgreementFramework1Choice,
    AgreementFramework1Code,
    AlternatePartyIdentification5,
    CashCollateral3,
    CashCollateral5,
    CollateralAccount1,
    CollateralAccountIdentificationType1Choice,
    CollateralAccountType1Code,
    CollateralOwnership1,
    CollateralSubstitution2,
    CollateralSubstitution3,
    CollateralSubstitutionRequestV03,
    CollateralSubstitutionSequence1Code,
    CollateralSubstitutionType1Code,
    ContactIdentification2,
    DateAndDateTimeChoice,
    DateCode9Choice,
    DateFormat14Choice,
    DateType2Code,
    DeliveringPartiesAndAccount10,
    DepositType1Code,
    Document,
    ExposureType5Code,
    FinancialInstrumentQuantity1Choice,
    GenericAccountIdentification1,
    GenericIdentification29,
    GenericIdentification30,
    GenericIdentification40,
    GenericIdentification58,
    IdentificationSource3Choice,
    IdentificationType40Choice,
    NameAndAddress5,
    NameAndAddress6,
    NameAndAddress13,
    NamePrefix1Code,
    Obligation3,
    OtherCollateral2,
    OtherCollateral4,
    OtherIdentification1,
    OtherTypeOfCollateral2,
    PartyIdentification32Choice,
    PartyIdentification33Choice,
    PartyIdentification34Choice,
    PartyIdentificationAndAccount77,
    PartyIdentificationAndAccount102,
    PartyTextInformation1,
    PostalAddress1,
    PostalAddress2,
    PostalAddress8,
    Price2,
    PriceRateOrAmountChoice,
    PriceValueType1Code,
    ReceivingPartiesAndAccount10,
    Reference17,
    SafekeepingPlace1Code,
    SafekeepingPlace3Code,
    SafekeepingPlaceFormat7Choice,
    SafekeepingPlaceTypeAndAnyBicidentifier1,
    SafekeepingPlaceTypeAndText1,
    SecuritiesAccount19,
    SecuritiesCollateral3,
    SecuritiesCollateral4,
    SecurityIdentification14,
    SettlementDetails88,
    SettlementParties3Choice,
    SimpleIdentificationInformation4,
    SubAccount4,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
    TypeOfIdentification2Code,
    YieldedOrValueType1Choice,
)

__all__ = [
    "AccountIdentification26",
    "AccountIdentification4Choice",
    "AccountSchemeName1Choice",
    "ActiveCurrencyAndAmount",
    "ActiveOrHistoricCurrencyAnd13DecimalAmount",
    "AddressType2Code",
    "Agreement2",
    "AgreementFramework1Choice",
    "AgreementFramework1Code",
    "AlternatePartyIdentification5",
    "CashCollateral3",
    "CashCollateral5",
    "CollateralAccount1",
    "CollateralAccountIdentificationType1Choice",
    "CollateralAccountType1Code",
    "CollateralOwnership1",
    "CollateralSubstitution2",
    "CollateralSubstitution3",
    "CollateralSubstitutionRequestV03",
    "CollateralSubstitutionSequence1Code",
    "CollateralSubstitutionType1Code",
    "ContactIdentification2",
    "DateAndDateTimeChoice",
    "DateCode9Choice",
    "DateFormat14Choice",
    "DateType2Code",
    "DeliveringPartiesAndAccount10",
    "DepositType1Code",
    "Document",
    "ExposureType5Code",
    "FinancialInstrumentQuantity1Choice",
    "GenericAccountIdentification1",
    "GenericIdentification29",
    "GenericIdentification30",
    "GenericIdentification40",
    "GenericIdentification58",
    "IdentificationSource3Choice",
    "IdentificationType40Choice",
    "NameAndAddress13",
    "NameAndAddress5",
    "NameAndAddress6",
    "NamePrefix1Code",
    "Obligation3",
    "OtherCollateral2",
    "OtherCollateral4",
    "OtherIdentification1",
    "OtherTypeOfCollateral2",
    "PartyIdentification32Choice",
    "PartyIdentification33Choice",
    "PartyIdentification34Choice",
    "PartyIdentificationAndAccount102",
    "PartyIdentificationAndAccount77",
    "PartyTextInformation1",
    "PostalAddress1",
    "PostalAddress2",
    "PostalAddress8",
    "Price2",
    "PriceRateOrAmountChoice",
    "PriceValueType1Code",
    "ReceivingPartiesAndAccount10",
    "Reference17",
    "SafekeepingPlace1Code",
    "SafekeepingPlace3Code",
    "SafekeepingPlaceFormat7Choice",
    "SafekeepingPlaceTypeAndAnyBicidentifier1",
    "SafekeepingPlaceTypeAndText1",
    "SecuritiesAccount19",
    "SecuritiesCollateral3",
    "SecuritiesCollateral4",
    "SecurityIdentification14",
    "SettlementDetails88",
    "SettlementParties3Choice",
    "SimpleIdentificationInformation4",
    "SubAccount4",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
    "TypeOfIdentification2Code",
    "YieldedOrValueType1Choice",
]



from __future__ import annotations

from dataclasses import dataclass, field
from enum import Enum

from xsdata.models.datatype import XmlDateTime

__NAMESPACE__ = "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01"


@dataclass(kw_only=True)
class AccountSchemeName1Choice:
    cd: None | str = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 4,
        },
    )
    prtry: None | str = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


class AddressType2Code(Enum):
    ADDR = "ADDR"
    PBOX = "PBOX"
    HOME = "HOME"
    BIZZ = "BIZZ"
    MLTO = "MLTO"
    DLVY = "DLVY"


@dataclass(kw_only=True)
class ClearingSystemIdentification2Choice:
    cd: None | str = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 5,
        },
    )
    prtry: None | str = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class CollateralValueReturnCriteria1:
    csh_acct_ownr_ind: None | bool = field(
        default=None,
        metadata={
            "name": "CshAcctOwnrInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    csh_acct_svcr_ind: None | bool = field(
        default=None,
        metadata={
            "name": "CshAcctSvcrInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    scties_acct_ownr_ind: None | bool = field(
        default=None,
        metadata={
            "name": "SctiesAcctOwnrInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    scties_acct_svcr_ind: None | bool = field(
        default=None,
        metadata={
            "name": "SctiesAcctSvcrInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    ttl_coll_valtn_ind: None | bool = field(
        default=None,
        metadata={
            "name": "TtlCollValtnInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    scties_acct_ind: None | bool = field(
        default=None,
        metadata={
            "name": "SctiesAcctInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    scties: None | bool = field(
        default=None,
        metadata={
            "name": "Scties",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class FinancialIdentificationSchemeName1Choice:
    cd: None | str = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 4,
        },
    )
    prtry: None | str = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class GenericIdentification1:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | str = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    issr: None | str = field(
        default=None,
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class GenericIdentification30:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "pattern": r"[a-zA-Z0-9]{4}",
        }
    )
    issr: str = field(
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | str = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class GenericIdentification36:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    issr: str = field(
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | str = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class IdentificationSource3Choice:
    cd: None | str = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 4,
        },
    )
    prtry: None | str = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class OriginalBusinessQuery1:
    msg_id: str = field(
        metadata={
            "name": "MsgId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    msg_nm_id: None | str = field(
        default=None,
        metadata={
            "name": "MsgNmId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    cre_dt_tm: None | XmlDateTime = field(
        default=None,
        metadata={
            "name": "CreDtTm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


class RequestType1Code(Enum):
    RT01 = "RT01"
    RT02 = "RT02"
    RT03 = "RT03"
    RT04 = "RT04"
    RT05 = "RT05"


class RequestType2Code(Enum):
    RT11 = "RT11"
    RT12 = "RT12"
    RT13 = "RT13"
    RT14 = "RT14"
    RT15 = "RT15"


@dataclass(kw_only=True)
class SupplementaryDataEnvelope1:
    any_element: None | object = field(
        default=None,
        metadata={
            "type": "Wildcard",
            "namespace": "##any",
        },
    )


@dataclass(kw_only=True)
class AddressType3Choice:
    cd: None | AddressType2Code = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    prtry: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class ClearingSystemMemberIdentification2:
    clr_sys_id: None | ClearingSystemIdentification2Choice = field(
        default=None,
        metadata={
            "name": "ClrSysId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    mmb_id: str = field(
        metadata={
            "name": "MmbId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )


@dataclass(kw_only=True)
class GenericAccountIdentification1:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 34,
        }
    )
    schme_nm: None | AccountSchemeName1Choice = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    issr: None | str = field(
        default=None,
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class GenericFinancialIdentification1:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | FinancialIdentificationSchemeName1Choice = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    issr: None | str = field(
        default=None,
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class OtherIdentification1:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    sfx: None | str = field(
        default=None,
        metadata={
            "name": "Sfx",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 16,
        },
    )
    tp: IdentificationSource3Choice = field(
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class PostalAddress1:
    adr_tp: None | AddressType2Code = field(
        default=None,
        metadata={
            "name": "AdrTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    adr_line: list[str] = field(
        default_factory=list,
        metadata={
            "name": "AdrLine",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "max_occurs": 5,
            "min_length": 1,
            "max_length": 70,
        },
    )
    strt_nm: None | str = field(
        default=None,
        metadata={
            "name": "StrtNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 70,
        },
    )
    bldg_nb: None | str = field(
        default=None,
        metadata={
            "name": "BldgNb",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 16,
        },
    )
    pst_cd: None | str = field(
        default=None,
        metadata={
            "name": "PstCd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 16,
        },
    )
    twn_nm: None | str = field(
        default=None,
        metadata={
            "name": "TwnNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    ctry_sub_dvsn: None | str = field(
        default=None,
        metadata={
            "name": "CtrySubDvsn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    ctry: str = field(
        metadata={
            "name": "Ctry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "pattern": r"[A-Z]{2,2}",
        }
    )


@dataclass(kw_only=True)
class RequestType2Choice:
    pmt_ctrl: None | RequestType1Code = field(
        default=None,
        metadata={
            "name": "PmtCtrl",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    enqry: None | RequestType2Code = field(
        default=None,
        metadata={
            "name": "Enqry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    prtry: None | GenericIdentification1 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class SupplementaryData1:
    plc_and_nm: None | str = field(
        default=None,
        metadata={
            "name": "PlcAndNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 350,
        },
    )
    envlp: SupplementaryDataEnvelope1 = field(
        metadata={
            "name": "Envlp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class AccountIdentification4Choice:
    iban: None | str = field(
        default=None,
        metadata={
            "name": "IBAN",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "pattern": r"[A-Z]{2,2}[0-9]{2,2}[a-zA-Z0-9]{1,30}",
        },
    )
    othr: None | GenericAccountIdentification1 = field(
        default=None,
        metadata={
            "name": "Othr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class MessageHeader3:
    msg_id: str = field(
        metadata={
            "name": "MsgId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    cre_dt_tm: None | XmlDateTime = field(
        default=None,
        metadata={
            "name": "CreDtTm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    req_tp: None | RequestType2Choice = field(
        default=None,
        metadata={
            "name": "ReqTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    orgnl_biz_qry: None | OriginalBusinessQuery1 = field(
        default=None,
        metadata={
            "name": "OrgnlBizQry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    qry_nm: None | str = field(
        default=None,
        metadata={
            "name": "QryNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class NameAndAddress5:
    nm: str = field(
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 350,
        }
    )
    adr: None | PostalAddress1 = field(
        default=None,
        metadata={
            "name": "Adr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class PostalAddress24:
    adr_tp: None | AddressType3Choice = field(
        default=None,
        metadata={
            "name": "AdrTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    dept: None | str = field(
        default=None,
        metadata={
            "name": "Dept",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 70,
        },
    )
    sub_dept: None | str = field(
        default=None,
        metadata={
            "name": "SubDept",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 70,
        },
    )
    strt_nm: None | str = field(
        default=None,
        metadata={
            "name": "StrtNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 70,
        },
    )
    bldg_nb: None | str = field(
        default=None,
        metadata={
            "name": "BldgNb",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 16,
        },
    )
    bldg_nm: None | str = field(
        default=None,
        metadata={
            "name": "BldgNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    flr: None | str = field(
        default=None,
        metadata={
            "name": "Flr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 70,
        },
    )
    pst_bx: None | str = field(
        default=None,
        metadata={
            "name": "PstBx",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 16,
        },
    )
    room: None | str = field(
        default=None,
        metadata={
            "name": "Room",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 70,
        },
    )
    pst_cd: None | str = field(
        default=None,
        metadata={
            "name": "PstCd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 16,
        },
    )
    twn_nm: None | str = field(
        default=None,
        metadata={
            "name": "TwnNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    twn_lctn_nm: None | str = field(
        default=None,
        metadata={
            "name": "TwnLctnNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    dstrct_nm: None | str = field(
        default=None,
        metadata={
            "name": "DstrctNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    ctry_sub_dvsn: None | str = field(
        default=None,
        metadata={
            "name": "CtrySubDvsn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    ctry: None | str = field(
        default=None,
        metadata={
            "name": "Ctry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "pattern": r"[A-Z]{2,2}",
        },
    )
    adr_line: list[str] = field(
        default_factory=list,
        metadata={
            "name": "AdrLine",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "max_occurs": 7,
            "min_length": 1,
            "max_length": 70,
        },
    )


@dataclass(kw_only=True)
class SecurityIdentification19:
    isin: None | str = field(
        default=None,
        metadata={
            "name": "ISIN",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "pattern": r"[A-Z]{2,2}[A-Z0-9]{9,9}[0-9]{1,1}",
        },
    )
    othr_id: list[OtherIdentification1] = field(
        default_factory=list,
        metadata={
            "name": "OthrId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    desc: None | str = field(
        default=None,
        metadata={
            "name": "Desc",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 140,
        },
    )


@dataclass(kw_only=True)
class BranchData3:
    id: None | str = field(
        default=None,
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    lei: None | str = field(
        default=None,
        metadata={
            "name": "LEI",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "pattern": r"[A-Z0-9]{18,18}[0-9]{2,2}",
        },
    )
    nm: None | str = field(
        default=None,
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 140,
        },
    )
    pstl_adr: None | PostalAddress24 = field(
        default=None,
        metadata={
            "name": "PstlAdr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class FinancialInstitutionIdentification18:
    bicfi: None | str = field(
        default=None,
        metadata={
            "name": "BICFI",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "pattern": r"[A-Z0-9]{4,4}[A-Z]{2,2}[A-Z0-9]{2,2}([A-Z0-9]{3,3}){0,1}",
        },
    )
    clr_sys_mmb_id: None | ClearingSystemMemberIdentification2 = field(
        default=None,
        metadata={
            "name": "ClrSysMmbId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    lei: None | str = field(
        default=None,
        metadata={
            "name": "LEI",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "pattern": r"[A-Z0-9]{18,18}[0-9]{2,2}",
        },
    )
    nm: None | str = field(
        default=None,
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 140,
        },
    )
    pstl_adr: None | PostalAddress24 = field(
        default=None,
        metadata={
            "name": "PstlAdr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    othr: None | GenericFinancialIdentification1 = field(
        default=None,
        metadata={
            "name": "Othr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class PartyIdentification120Choice:
    any_bic: None | str = field(
        default=None,
        metadata={
            "name": "AnyBIC",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "pattern": r"[A-Z0-9]{4,4}[A-Z]{2,2}[A-Z0-9]{2,2}([A-Z0-9]{3,3}){0,1}",
        },
    )
    prtry_id: None | GenericIdentification36 = field(
        default=None,
        metadata={
            "name": "PrtryId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    nm_and_adr: None | NameAndAddress5 = field(
        default=None,
        metadata={
            "name": "NmAndAdr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class BranchAndFinancialInstitutionIdentification6:
    fin_instn_id: FinancialInstitutionIdentification18 = field(
        metadata={
            "name": "FinInstnId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
        }
    )
    brnch_id: None | BranchData3 = field(
        default=None,
        metadata={
            "name": "BrnchId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class PartyIdentification136:
    id: PartyIdentification120Choice = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
        }
    )
    lei: None | str = field(
        default=None,
        metadata={
            "name": "LEI",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "pattern": r"[A-Z0-9]{18,18}[0-9]{2,2}",
        },
    )


@dataclass(kw_only=True)
class SystemPartyIdentification8:
    id: PartyIdentification136 = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
        }
    )
    rspnsbl_pty_id: None | PartyIdentification136 = field(
        default=None,
        metadata={
            "name": "RspnsblPtyId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class CollateralValueSearchCriteria3:
    csh_acct_id: None | AccountIdentification4Choice = field(
        default=None,
        metadata={
            "name": "CshAcctId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    ccy: list[str] = field(
        default_factory=list,
        metadata={
            "name": "Ccy",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "pattern": r"[A-Z]{3,3}",
        },
    )
    csh_acct_ownr: None | SystemPartyIdentification8 = field(
        default=None,
        metadata={
            "name": "CshAcctOwnr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    csh_acct_svcr: None | BranchAndFinancialInstitutionIdentification6 = field(
        default=None,
        metadata={
            "name": "CshAcctSvcr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    fin_instrm_id: list[SecurityIdentification19] = field(
        default_factory=list,
        metadata={
            "name": "FinInstrmId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    scties_acct_ownr: None | SystemPartyIdentification8 = field(
        default=None,
        metadata={
            "name": "SctiesAcctOwnr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    scties_acct_svcr: None | PartyIdentification136 = field(
        default=None,
        metadata={
            "name": "SctiesAcctSvcr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class CollateralValueCriteria3:
    qry_nm: None | str = field(
        default=None,
        metadata={
            "name": "QryNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    sch_crit: None | CollateralValueSearchCriteria3 = field(
        default=None,
        metadata={
            "name": "SchCrit",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    rtr_crit: None | CollateralValueReturnCriteria1 = field(
        default=None,
        metadata={
            "name": "RtrCrit",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class CollateralValueCriteriaDefinition3Choice:
    qry_nm: None | str = field(
        default=None,
        metadata={
            "name": "QryNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "min_length": 1,
            "max_length": 35,
        },
    )
    new_crit: None | CollateralValueCriteria3 = field(
        default=None,
        metadata={
            "name": "NewCrit",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class CollateralValueQueryV01:
    msg_hdr: MessageHeader3 = field(
        metadata={
            "name": "MsgHdr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
            "required": True,
        }
    )
    coll_val_qry_def: None | CollateralValueCriteriaDefinition3Choice = field(
        default=None,
        metadata={
            "name": "CollValQryDef",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )
    splmtry_data: list[SupplementaryData1] = field(
        default_factory=list,
        metadata={
            "name": "SplmtryData",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01",
        },
    )


@dataclass(kw_only=True)
class Document:
    class Meta:
        namespace = "urn:iso:std:iso:20022:tech:xsd:colr.001.001.01"

    coll_val_qry: CollateralValueQueryV01 = field(
        metadata={
            "name": "CollValQry",
            "type": "Element",
            "required": True,
        }
    )



from pyiso20022.admi.admi_007_001_01.admi_007_001_01 import (
    AddressType2Code,
    Document,
    GenericIdentification36,
    MessageHeader10,
    MessageReference1,
    NameAndAddress5,
    PartyIdentification120Choice,
    PartyIdentification136,
    PostalAddress1,
    ReceiptAcknowledgementReport2,
    ReceiptAcknowledgementV01,
    RequestHandling2,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
)

__all__ = [
    "AddressType2Code",
    "Document",
    "GenericIdentification36",
    "MessageHeader10",
    "MessageReference1",
    "NameAndAddress5",
    "PartyIdentification120Choice",
    "PartyIdentification136",
    "PostalAddress1",
    "ReceiptAcknowledgementReport2",
    "ReceiptAcknowledgementV01",
    "RequestHandling2",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
]



from pyiso20022.colr.colr_004_001_04.colr_004_001_04 import (
    ActiveCurrencyAndAmount,
    AggregatedIndependentAmount1,
    AgreedAmount1,
    AgreedAmount1Choice,
    Agreement4,
    AgreementFramework1Choice,
    AgreementFramework1Code,
    Amount1,
    Collateral1,
    CollateralAccount2,
    CollateralAccountIdentificationType2Choice,
    CollateralAccountType1Code,
    CollateralBalance1Choice,
    DateAndDateTimeChoice,
    Document,
    ExposureConventionType1Code,
    ExposureType5Code,
    GenericIdentification30,
    GenericIdentification36,
    IndependentAmount1,
    IndependentAmount2,
    IndependentAmountConventionType1Code,
    Margin1,
    MarginCall1,
    MarginCallResponse1Code,
    MarginCallResponseV04,
    MarginCollateral1,
    MarginTerms1Choice,
    NameAndAddress6,
    Obligation4,
    PartyIdentification100Choice,
    PostalAddress2,
    Response1,
    ResponseType1Choice,
    RoundingMethod1Code,
    SegregatedIndependentAmountMargin1,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
    ThresholdType1Code,
    VariationMargin1,
)

__all__ = [
    "ActiveCurrencyAndAmount",
    "AggregatedIndependentAmount1",
    "AgreedAmount1",
    "AgreedAmount1Choice",
    "Agreement4",
    "AgreementFramework1Choice",
    "AgreementFramework1Code",
    "Amount1",
    "Collateral1",
    "CollateralAccount2",
    "CollateralAccountIdentificationType2Choice",
    "CollateralAccountType1Code",
    "CollateralBalance1Choice",
    "DateAndDateTimeChoice",
    "Document",
    "ExposureConventionType1Code",
    "ExposureType5Code",
    "GenericIdentification30",
    "GenericIdentification36",
    "IndependentAmount1",
    "IndependentAmount2",
    "IndependentAmountConventionType1Code",
    "Margin1",
    "MarginCall1",
    "MarginCallResponse1Code",
    "MarginCallResponseV04",
    "MarginCollateral1",
    "MarginTerms1Choice",
    "NameAndAddress6",
    "Obligation4",
    "PartyIdentification100Choice",
    "PostalAddress2",
    "Response1",
    "ResponseType1Choice",
    "RoundingMethod1Code",
    "SegregatedIndependentAmountMargin1",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
    "ThresholdType1Code",
    "VariationMargin1",
]



from __future__ import annotations

from dataclasses import dataclass, field
from decimal import Decimal
from enum import Enum

from xsdata.models.datatype import XmlDate, XmlDateTime

__NAMESPACE__ = "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05"


@dataclass(kw_only=True)
class ActiveCurrencyAndAmount:
    value: Decimal = field(
        metadata={
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 18,
            "fraction_digits": 5,
        }
    )
    ccy: str = field(
        metadata={
            "name": "Ccy",
            "type": "Attribute",
            "required": True,
            "pattern": r"[A-Z]{3,3}",
        }
    )


@dataclass(kw_only=True)
class ActiveOrHistoricCurrencyAnd13DecimalAmount:
    value: Decimal = field(
        metadata={
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 18,
            "fraction_digits": 13,
        }
    )
    ccy: str = field(
        metadata={
            "name": "Ccy",
            "type": "Attribute",
            "required": True,
            "pattern": r"[A-Z]{3,3}",
        }
    )


class AgreementFramework1Code(Enum):
    FBAA = "FBAA"
    BBAA = "BBAA"
    DERV = "DERV"
    ISDA = "ISDA"
    NONR = "NONR"


class CcpmemberType1Code(Enum):
    ACMB = "ACMB"
    CCPX = "CCPX"
    DCMB = "DCMB"
    FCMC = "FCMC"
    GCMB = "GCMB"
    SCMB = "SCMB"


class CollateralAccountType1Code(Enum):
    HOUS = "HOUS"
    CLIE = "CLIE"
    LIPR = "LIPR"
    MGIN = "MGIN"
    DFLT = "DFLT"


class CollateralAppliedExcess1Code(Enum):
    APLD = "APLD"
    EXCS = "EXCS"


class CollateralDirection1Code(Enum):
    CDPA = "CDPA"
    CDPB = "CDPB"


class CollateralType8Code(Enum):
    COMO = "COMO"
    CCCL = "CCCL"
    CEMC = "CEMC"
    CXCC = "CXCC"
    CFTD = "CFTD"
    CFTI = "CFTI"
    CTRC = "CTRC"
    CASH = "CASH"
    LCRE = "LCRE"
    OTHR = "OTHR"
    SECU = "SECU"
    CTCO = "CTCO"
    CCVR = "CCVR"


@dataclass(kw_only=True)
class DateAndDateTime2Choice:
    dt: None | XmlDate = field(
        default=None,
        metadata={
            "name": "Dt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    dt_tm: None | XmlDateTime = field(
        default=None,
        metadata={
            "name": "DtTm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


class DateType2Code(Enum):
    OPEN = "OPEN"


class DepositType1Code(Enum):
    FITE = "FITE"
    CALL = "CALL"


class EventFrequency6Code(Enum):
    DAIL = "DAIL"
    INDA = "INDA"
    ONDE = "ONDE"


class ExposureType11Code(Enum):
    BFWD = "BFWD"
    PAYM = "PAYM"
    CBCO = "CBCO"
    COMM = "COMM"
    CRDS = "CRDS"
    CRTL = "CRTL"
    CRSP = "CRSP"
    CCIR = "CCIR"
    CRPR = "CRPR"
    EQPT = "EQPT"
    EQUS = "EQUS"
    EXTD = "EXTD"
    EXPT = "EXPT"
    FIXI = "FIXI"
    FORX = "FORX"
    FORW = "FORW"
    FUTR = "FUTR"
    OPTN = "OPTN"
    LIQU = "LIQU"
    OTCD = "OTCD"
    RVPO = "RVPO"
    SLOA = "SLOA"
    SBSC = "SBSC"
    SCRP = "SCRP"
    SLEB = "SLEB"
    SCIR = "SCIR"
    SCIE = "SCIE"
    SWPT = "SWPT"
    TBAS = "TBAS"
    TRCP = "TRCP"
    UDMS = "UDMS"
    CCPC = "CCPC"
    EQUI = "EQUI"
    TRBD = "TRBD"
    REPO = "REPO"
    SHSL = "SHSL"


class ExposureType13Code(Enum):
    CCIR = "CCIR"
    CRPR = "CRPR"
    EQUI = "EQUI"
    EQPT = "EQPT"
    EQUS = "EQUS"
    EXTD = "EXTD"
    EXPT = "EXPT"
    FIXI = "FIXI"
    FORX = "FORX"
    FORW = "FORW"
    FUTR = "FUTR"
    OPTN = "OPTN"
    LIQU = "LIQU"
    MGLD = "MGLD"
    OTCD = "OTCD"
    REPO = "REPO"
    RVPO = "RVPO"
    SLOA = "SLOA"
    SBSC = "SBSC"
    SCRP = "SCRP"
    SLEB = "SLEB"
    SHSL = "SHSL"
    SCIR = "SCIR"
    SCIE = "SCIE"
    ESCL = "ESCL"
    SWPT = "SWPT"
    TBAS = "TBAS"
    ECRT = "ECRT"
    ECFR = "ECFR"
    EMLO = "EMLO"
    EMLI = "EMLI"
    EOIM = "EOIM"
    EOMI = "EOMI"
    TRBD = "TRBD"
    BFWD = "BFWD"
    PAYM = "PAYM"
    CCPC = "CCPC"
    COMM = "COMM"
    CRDS = "CRDS"
    CRTL = "CRTL"
    CRSP = "CRSP"
    EOMO = "EOMO"
    CBCO = "CBCO"
    TRCP = "TRCP"
    UDMS = "UDMS"


@dataclass(kw_only=True)
class FinancialInstrumentQuantity33Choice:
    unit: None | Decimal = field(
        default=None,
        metadata={
            "name": "Unit",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 18,
            "fraction_digits": 17,
        },
    )
    face_amt: None | Decimal = field(
        default=None,
        metadata={
            "name": "FaceAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_inclusive": Decimal("0"),
            "total_digits": 18,
            "fraction_digits": 5,
        },
    )
    amtsd_val: None | Decimal = field(
        default=None,
        metadata={
            "name": "AmtsdVal",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_inclusive": Decimal("0"),
            "total_digits": 18,
            "fraction_digits": 5,
        },
    )
    dgtl_tkn_unit: None | Decimal = field(
        default=None,
        metadata={
            "name": "DgtlTknUnit",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 30,
            "fraction_digits": 29,
        },
    )


@dataclass(kw_only=True)
class GenericIdentification30:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "pattern": r"[a-zA-Z0-9]{4}",
        }
    )
    issr: str = field(
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | str = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class GenericIdentification36:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    issr: str = field(
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | str = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class IdentificationSource3Choice:
    cd: None | str = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 4,
        },
    )
    prtry: None | str = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )


class InterestComputationMethod2Code(Enum):
    A001 = "A001"
    A002 = "A002"
    A003 = "A003"
    A004 = "A004"
    A005 = "A005"
    A006 = "A006"
    A007 = "A007"
    A008 = "A008"
    A009 = "A009"
    A010 = "A010"
    A011 = "A011"
    A012 = "A012"
    A013 = "A013"
    A014 = "A014"
    NARR = "NARR"


@dataclass(kw_only=True)
class Pagination1:
    pg_nb: str = field(
        metadata={
            "name": "PgNb",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "pattern": r"[0-9]{1,5}",
        }
    )
    last_pg_ind: bool = field(
        metadata={
            "name": "LastPgInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class PostalAddress2:
    strt_nm: None | str = field(
        default=None,
        metadata={
            "name": "StrtNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 70,
        },
    )
    pst_cd_id: str = field(
        metadata={
            "name": "PstCdId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 16,
        }
    )
    twn_nm: str = field(
        metadata={
            "name": "TwnNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    ctry_sub_dvsn: None | str = field(
        default=None,
        metadata={
            "name": "CtrySubDvsn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    ctry: str = field(
        metadata={
            "name": "Ctry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "pattern": r"[A-Z]{2,2}",
        }
    )


class PriceValueType1Code(Enum):
    DISC = "DISC"
    PREM = "PREM"
    PARV = "PARV"


class ReturnExcessCash1Code(Enum):
    RTND = "RTND"
    RTDN = "RTDN"
    SSPD = "SSPD"


class SafekeepingPlace1Code(Enum):
    CUST = "CUST"
    ICSD = "ICSD"
    NCSD = "NCSD"
    SHHE = "SHHE"


class SafekeepingPlace3Code(Enum):
    SHHE = "SHHE"


class SettlementStatus3Code(Enum):
    ASTL = "ASTL"
    AAUT = "AAUT"
    ACCF = "ACCF"
    ARCF = "ARCF"
    MTCH = "MTCH"
    PSTL = "PSTL"
    RJCT = "RJCT"
    STLD = "STLD"
    STCR = "STCR"
    SPLT = "SPLT"
    NMAT = "NMAT"


class ShortLong1Code(Enum):
    SHOR = "SHOR"
    LONG = "LONG"


@dataclass(kw_only=True)
class SupplementaryDataEnvelope1:
    any_element: None | object = field(
        default=None,
        metadata={
            "type": "Wildcard",
            "namespace": "##any",
        },
    )


class ThresholdType1Code(Enum):
    SECU = "SECU"
    UNSE = "UNSE"


@dataclass(kw_only=True)
class AgreementFramework1Choice:
    agrmt_frmwk: None | AgreementFramework1Code = field(
        default=None,
        metadata={
            "name": "AgrmtFrmwk",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    prtry_id: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "PrtryId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class BlockChainAddressWallet3:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 140,
        }
    )
    tp: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    nm: None | str = field(
        default=None,
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 70,
        },
    )


@dataclass(kw_only=True)
class CashCollateral4:
    asst_nb: None | str = field(
        default=None,
        metadata={
            "name": "AsstNb",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    dpst_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "DpstAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    dpst_tp: None | DepositType1Code = field(
        default=None,
        metadata={
            "name": "DpstTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    blckd_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "BlckdAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    mtrty_dt: None | XmlDate = field(
        default=None,
        metadata={
            "name": "MtrtyDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    val_dt: None | XmlDate = field(
        default=None,
        metadata={
            "name": "ValDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    xchg_rate: None | Decimal = field(
        default=None,
        metadata={
            "name": "XchgRate",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    coll_val: ActiveCurrencyAndAmount = field(
        metadata={
            "name": "CollVal",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    hrcut: None | Decimal = field(
        default=None,
        metadata={
            "name": "Hrcut",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )


@dataclass(kw_only=True)
class CollateralAccountIdentificationType3Choice:
    tp: None | CollateralAccountType1Code = field(
        default=None,
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    prtry: None | GenericIdentification36 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class CollateralAmount1:
    coll_amt: ActiveCurrencyAndAmount = field(
        metadata={
            "name": "CollAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    rptd_ccy_and_amt: ActiveCurrencyAndAmount = field(
        metadata={
            "name": "RptdCcyAndAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    mkt_val_amt: ActiveCurrencyAndAmount = field(
        metadata={
            "name": "MktValAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    acrd_intrst_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "AcrdIntrstAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    fees_and_comssns: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "FeesAndComssns",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class DateCode9Choice:
    cd: None | DateType2Code = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    prtry: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class GenericIdentification78:
    tp: GenericIdentification30 = field(
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    id: None | str = field(
        default=None,
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class NameAndAddress6:
    nm: str = field(
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 70,
        }
    )
    adr: PostalAddress2 = field(
        metadata={
            "name": "Adr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class OtherIdentification1:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    sfx: None | str = field(
        default=None,
        metadata={
            "name": "Sfx",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 16,
        },
    )
    tp: IdentificationSource3Choice = field(
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class OtherTypeOfCollateral3:
    desc: str = field(
        metadata={
            "name": "Desc",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 140,
        }
    )
    qty: None | FinancialInstrumentQuantity33Choice = field(
        default=None,
        metadata={
            "name": "Qty",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class PriceRateOrAmount3Choice:
    rate: None | Decimal = field(
        default=None,
        metadata={
            "name": "Rate",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    amt: None | ActiveOrHistoricCurrencyAnd13DecimalAmount = field(
        default=None,
        metadata={
            "name": "Amt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class ReportParameters6:
    rpt_id: str = field(
        metadata={
            "name": "RptId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    rpt_dt_and_tm: DateAndDateTime2Choice = field(
        metadata={
            "name": "RptDtAndTm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    frqcy: EventFrequency6Code = field(
        metadata={
            "name": "Frqcy",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    rpt_ccy: str = field(
        metadata={
            "name": "RptCcy",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "pattern": r"[A-Z]{3,3}",
        }
    )
    clctn_dt: None | XmlDateTime = field(
        default=None,
        metadata={
            "name": "ClctnDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class ReturnExcessCash1Choice:
    cd: None | ReturnExcessCash1Code = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    prtry: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class SafekeepingPlaceTypeAndIdentification1:
    sfkpg_plc_tp: SafekeepingPlace1Code = field(
        metadata={
            "name": "SfkpgPlcTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "pattern": r"[A-Z0-9]{4,4}[A-Z]{2,2}[A-Z0-9]{2,2}([A-Z0-9]{3,3}){0,1}",
        }
    )


@dataclass(kw_only=True)
class SafekeepingPlaceTypeAndText8:
    sfkpg_plc_tp: SafekeepingPlace3Code = field(
        metadata={
            "name": "SfkpgPlcTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    id: None | str = field(
        default=None,
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class SecuritiesAccount19:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    tp: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    nm: None | str = field(
        default=None,
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 70,
        },
    )


@dataclass(kw_only=True)
class SupplementaryData1:
    plc_and_nm: None | str = field(
        default=None,
        metadata={
            "name": "PlcAndNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 350,
        },
    )
    envlp: SupplementaryDataEnvelope1 = field(
        metadata={
            "name": "Envlp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class YieldedOrValueType1Choice:
    yldd: None | bool = field(
        default=None,
        metadata={
            "name": "Yldd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    val_tp: None | PriceValueType1Code = field(
        default=None,
        metadata={
            "name": "ValTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class Agreement4:
    agrmt_dtls: str = field(
        metadata={
            "name": "AgrmtDtls",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 140,
        }
    )
    agrmt_id: None | str = field(
        default=None,
        metadata={
            "name": "AgrmtId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 140,
        },
    )
    agrmt_dt: XmlDate = field(
        metadata={
            "name": "AgrmtDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    base_ccy: str = field(
        metadata={
            "name": "BaseCcy",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "pattern": r"[A-Z]{3,3}",
        }
    )
    agrmt_frmwk: None | AgreementFramework1Choice = field(
        default=None,
        metadata={
            "name": "AgrmtFrmwk",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class BlockChainAddressWallet5:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 140,
        }
    )
    tp: None | CollateralAccountIdentificationType3Choice = field(
        default=None,
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    nm: None | str = field(
        default=None,
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 70,
        },
    )


@dataclass(kw_only=True)
class CollateralAccount3:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    tp: None | CollateralAccountIdentificationType3Choice = field(
        default=None,
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    nm: None | str = field(
        default=None,
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 70,
        },
    )


@dataclass(kw_only=True)
class DateFormat14Choice:
    dt: None | XmlDate = field(
        default=None,
        metadata={
            "name": "Dt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    dt_cd: None | DateCode9Choice = field(
        default=None,
        metadata={
            "name": "DtCd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class PartyIdentification178Choice:
    any_bic: None | str = field(
        default=None,
        metadata={
            "name": "AnyBIC",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "pattern": r"[A-Z0-9]{4,4}[A-Z]{2,2}[A-Z0-9]{2,2}([A-Z0-9]{3,3}){0,1}",
        },
    )
    prtry_id: None | GenericIdentification36 = field(
        default=None,
        metadata={
            "name": "PrtryId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    nm_and_adr: None | NameAndAddress6 = field(
        default=None,
        metadata={
            "name": "NmAndAdr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class Price7:
    tp: YieldedOrValueType1Choice = field(
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    val: PriceRateOrAmount3Choice = field(
        metadata={
            "name": "Val",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class ReturnExcessCash1:
    rtr_xcss_csh_tp: ReturnExcessCash1Choice = field(
        metadata={
            "name": "RtrXcssCshTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    csh_coll_ccy: str = field(
        metadata={
            "name": "CshCollCcy",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
            "pattern": r"[A-Z]{3,3}",
        }
    )


@dataclass(kw_only=True)
class SafekeepingPlaceFormat29Choice:
    id: None | SafekeepingPlaceTypeAndText8 = field(
        default=None,
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    ctry: None | str = field(
        default=None,
        metadata={
            "name": "Ctry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "pattern": r"[A-Z]{2,2}",
        },
    )
    tp_and_id: None | SafekeepingPlaceTypeAndIdentification1 = field(
        default=None,
        metadata={
            "name": "TpAndId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    prtry: None | GenericIdentification78 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class SecurityIdentification19:
    isin: None | str = field(
        default=None,
        metadata={
            "name": "ISIN",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "pattern": r"[A-Z]{2,2}[A-Z0-9]{9,9}[0-9]{1,1}",
        },
    )
    othr_id: list[OtherIdentification1] = field(
        default_factory=list,
        metadata={
            "name": "OthrId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    desc: None | str = field(
        default=None,
        metadata={
            "name": "Desc",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 140,
        },
    )


@dataclass(kw_only=True)
class CollateralOwnership3:
    prtry: bool = field(
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    clnt_nm: None | PartyIdentification178Choice = field(
        default=None,
        metadata={
            "name": "ClntNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class PartyIdentification242:
    id: PartyIdentification178Choice = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    ccpmmb_tp: None | CcpmemberType1Code = field(
        default=None,
        metadata={
            "name": "CCPMmbTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class SummaryAmounts2:
    thrshld_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "ThrshldAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    thrshld_tp: None | ThresholdType1Code = field(
        default=None,
        metadata={
            "name": "ThrshldTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    pre_hrcut_coll_val: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "PreHrcutCollVal",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    adjstd_xpsr: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "AdjstdXpsr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    coll_reqrd: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "CollReqrd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    rtr_xcss_csh_and_coll_ccy: list[ReturnExcessCash1] = field(
        default_factory=list,
        metadata={
            "name": "RtrXcssCshAndCollCcy",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    min_trf_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "MinTrfAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    rndg_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "RndgAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    prvs_xpsr_val: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "PrvsXpsrVal",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    prvs_coll_val: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "PrvsCollVal",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    ttl_pdg_incmg_coll: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "TtlPdgIncmgColl",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    ttl_pdg_outgng_coll: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "TtlPdgOutgngColl",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    ttl_acrd_intrst_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "TtlAcrdIntrstAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    ttl_fees: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "TtlFees",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class Obligation11:
    pty_a: PartyIdentification242 = field(
        metadata={
            "name": "PtyA",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    svcg_pty_a: None | PartyIdentification178Choice = field(
        default=None,
        metadata={
            "name": "SvcgPtyA",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    pty_b: PartyIdentification242 = field(
        metadata={
            "name": "PtyB",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    svcg_pty_b: None | PartyIdentification178Choice = field(
        default=None,
        metadata={
            "name": "SvcgPtyB",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    coll_acct_id: None | CollateralAccount3 = field(
        default=None,
        metadata={
            "name": "CollAcctId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    blck_chain_adr_or_wllt: None | BlockChainAddressWallet5 = field(
        default=None,
        metadata={
            "name": "BlckChainAdrOrWllt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    xpsr_tp: None | ExposureType11Code = field(
        default=None,
        metadata={
            "name": "XpsrTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    valtn_dt: DateAndDateTime2Choice = field(
        metadata={
            "name": "ValtnDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class OtherCollateral10:
    asst_nb: None | str = field(
        default=None,
        metadata={
            "name": "AsstNb",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    lttr_of_cdt_id: None | str = field(
        default=None,
        metadata={
            "name": "LttrOfCdtId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    lttr_of_cdt_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "LttrOfCdtAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    grnt_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "GrntAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    othr_tp_of_coll: None | OtherTypeOfCollateral3 = field(
        default=None,
        metadata={
            "name": "OthrTpOfColl",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    coll_ownrsh: None | CollateralOwnership3 = field(
        default=None,
        metadata={
            "name": "CollOwnrsh",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    isse_dt: None | DateFormat14Choice = field(
        default=None,
        metadata={
            "name": "IsseDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    xpry_dt: None | DateFormat14Choice = field(
        default=None,
        metadata={
            "name": "XpryDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    ltd_cvrg_ind: None | bool = field(
        default=None,
        metadata={
            "name": "LtdCvrgInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    issr: None | PartyIdentification178Choice = field(
        default=None,
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    blckd_qty: None | FinancialInstrumentQuantity33Choice = field(
        default=None,
        metadata={
            "name": "BlckdQty",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    val_dt: None | XmlDate = field(
        default=None,
        metadata={
            "name": "ValDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    xchg_rate: None | Decimal = field(
        default=None,
        metadata={
            "name": "XchgRate",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    mkt_val: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "MktVal",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    hrcut: None | Decimal = field(
        default=None,
        metadata={
            "name": "Hrcut",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    coll_val: ActiveCurrencyAndAmount = field(
        metadata={
            "name": "CollVal",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    sfkpg_plc: None | SafekeepingPlaceFormat29Choice = field(
        default=None,
        metadata={
            "name": "SfkpgPlc",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    sfkpg_acct: None | SecuritiesAccount19 = field(
        default=None,
        metadata={
            "name": "SfkpgAcct",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    blck_chain_adr_or_wllt: None | BlockChainAddressWallet3 = field(
        default=None,
        metadata={
            "name": "BlckChainAdrOrWllt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class SecuritiesCollateral13:
    asst_nb: None | str = field(
        default=None,
        metadata={
            "name": "AsstNb",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    scty_id: SecurityIdentification19 = field(
        metadata={
            "name": "SctyId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    mtrty_dt: None | DateAndDateTime2Choice = field(
        default=None,
        metadata={
            "name": "MtrtyDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    coll_ownrsh: None | CollateralOwnership3 = field(
        default=None,
        metadata={
            "name": "CollOwnrsh",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    ltd_cvrg_ind: None | bool = field(
        default=None,
        metadata={
            "name": "LtdCvrgInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    qty: FinancialInstrumentQuantity33Choice = field(
        metadata={
            "name": "Qty",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    blckd_qty: None | FinancialInstrumentQuantity33Choice = field(
        default=None,
        metadata={
            "name": "BlckdQty",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    pric: None | Price7 = field(
        default=None,
        metadata={
            "name": "Pric",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    mkt_val: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "MktVal",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    hrcut: None | Decimal = field(
        default=None,
        metadata={
            "name": "Hrcut",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    coll_val: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "CollVal",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    val_dt: None | XmlDate = field(
        default=None,
        metadata={
            "name": "ValDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    sfkpg_acct: None | SecuritiesAccount19 = field(
        default=None,
        metadata={
            "name": "SfkpgAcct",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    blck_chain_adr_or_wllt: None | BlockChainAddressWallet3 = field(
        default=None,
        metadata={
            "name": "BlckChainAdrOrWllt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    sfkpg_plc: SafekeepingPlaceFormat29Choice = field(
        metadata={
            "name": "SfkpgPlc",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class Summary3:
    xpsd_amt_pty_a: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "XpsdAmtPtyA",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    xpsd_amt_pty_b: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "XpsdAmtPtyB",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    xpsr_tp: ExposureType13Code = field(
        metadata={
            "name": "XpsrTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    ttl_val_of_coll: ActiveCurrencyAndAmount = field(
        metadata={
            "name": "TtlValOfColl",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    net_xcss_dfcit: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "NetXcssDfcit",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    net_xcss_dfcit_ind: None | ShortLong1Code = field(
        default=None,
        metadata={
            "name": "NetXcssDfcitInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    valtn_dt_tm: XmlDateTime = field(
        metadata={
            "name": "ValtnDtTm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    reqd_sttlm_dt: None | XmlDate = field(
        default=None,
        metadata={
            "name": "ReqdSttlmDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    summry_dtls: None | SummaryAmounts2 = field(
        default=None,
        metadata={
            "name": "SummryDtls",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class CollateralValuation13:
    coll_id: None | str = field(
        default=None,
        metadata={
            "name": "CollId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    coll_tp: CollateralType8Code = field(
        metadata={
            "name": "CollTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    coll_drctn: None | CollateralDirection1Code = field(
        default=None,
        metadata={
            "name": "CollDrctn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    sttlm_sts: SettlementStatus3Code = field(
        metadata={
            "name": "SttlmSts",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    apld_xcss_ind: None | CollateralAppliedExcess1Code = field(
        default=None,
        metadata={
            "name": "ApldXcssInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    nb_of_days_acrd: None | Decimal = field(
        default=None,
        metadata={
            "name": "NbOfDaysAcrd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 18,
            "fraction_digits": 0,
        },
    )
    valtn_amts: CollateralAmount1 = field(
        metadata={
            "name": "ValtnAmts",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    day_cnt_bsis: None | InterestComputationMethod2Code = field(
        default=None,
        metadata={
            "name": "DayCntBsis",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    xchg_rate: None | Decimal = field(
        default=None,
        metadata={
            "name": "XchgRate",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    ccy_hrcut: None | Decimal = field(
        default=None,
        metadata={
            "name": "CcyHrcut",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    adjstd_rate: None | Decimal = field(
        default=None,
        metadata={
            "name": "AdjstdRate",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    scties_coll: None | SecuritiesCollateral13 = field(
        default=None,
        metadata={
            "name": "SctiesColl",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    csh_coll: None | CashCollateral4 = field(
        default=None,
        metadata={
            "name": "CshColl",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    othr_coll: None | OtherCollateral10 = field(
        default=None,
        metadata={
            "name": "OthrColl",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class Collateral53:
    acct_id: None | CollateralAccount3 = field(
        default=None,
        metadata={
            "name": "AcctId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    blck_chain_adr_or_wllt: None | BlockChainAddressWallet5 = field(
        default=None,
        metadata={
            "name": "BlckChainAdrOrWllt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    rpt_summry: Summary3 = field(
        metadata={
            "name": "RptSummry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    coll_valtn: list[CollateralValuation13] = field(
        default_factory=list,
        metadata={
            "name": "CollValtn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class CollateralAndExposureReportV05:
    rpt_params: ReportParameters6 = field(
        metadata={
            "name": "RptParams",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    pgntn: None | Pagination1 = field(
        default=None,
        metadata={
            "name": "Pgntn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    oblgtn: Obligation11 = field(
        metadata={
            "name": "Oblgtn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "required": True,
        }
    )
    agrmt: None | Agreement4 = field(
        default=None,
        metadata={
            "name": "Agrmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )
    coll_rpt: list[Collateral53] = field(
        default_factory=list,
        metadata={
            "name": "CollRpt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
            "min_occurs": 1,
        },
    )
    splmtry_data: list[SupplementaryData1] = field(
        default_factory=list,
        metadata={
            "name": "SplmtryData",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05",
        },
    )


@dataclass(kw_only=True)
class Document:
    class Meta:
        namespace = "urn:iso:std:iso:20022:tech:xsd:colr.016.001.05"

    coll_and_xpsr_rpt: CollateralAndExposureReportV05 = field(
        metadata={
            "name": "CollAndXpsrRpt",
            "type": "Element",
            "required": True,
        }
    )



from pyiso20022.colr.colr_023_001_01.colr_023_001_01 import (
    ActiveCurrencyAndAmount,
    ActiveOrHistoricCurrencyAndAmount,
    AddressType2Code,
    AllocationStatus1Choice,
    AlternatePartyIdentification7,
    AmountAndDirection44,
    AmountAndDirection49,
    BlockChainAddressWallet3,
    CashAccountIdentification5Choice,
    CashMovement7,
    ClosingDate4Choice,
    CollateralAmount5,
    CollateralAmount14,
    CollateralDate2,
    CollateralParameters13,
    CollateralParties8,
    CollateralRole1Code,
    CollateralStatus3Choice,
    CollateralTransactionType1Choice,
    CollateralTransactionType1Code,
    CreditDebit3Code,
    CreditDebitCode,
    Date3Choice,
    DateAndDateTime2Choice,
    DateType2Code,
    DealTransactionDetails7,
    Document,
    ExposureType14Code,
    ExposureType23Choice,
    FinancialInstrumentQuantity33Choice,
    ForeignExchangeTerms23,
    GenericIdentification30,
    GenericIdentification36,
    IdentificationSource3Choice,
    IdentificationType42Choice,
    NameAndAddress5,
    OriginalAndCurrentQuantities1,
    OtherIdentification1,
    Pagination1,
    PartyIdentification120Choice,
    PartyIdentification136,
    PartyIdentificationAndAccount193,
    PartyIdentificationAndAccount202,
    PartyIdentificationAndAccount203,
    PostalAddress1,
    ProprietaryReason4,
    ProprietaryStatusAndReason6,
    Quantity51Choice,
    ReceiveDelivery1Code,
    SecuritiesAccount19,
    SecuritiesMovement8,
    SecuritiesMovementStatus1Choice,
    SecurityIdentification19,
    SettlementStatus27Choice,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
    TradingCapacity7Code,
    TradingPartyCapacity5Choice,
    TransactionIdentifications46,
    TripartyCollateralStatusAdviceV01,
    TypeOfIdentification1Code,
)

__all__ = [
    "ActiveCurrencyAndAmount",
    "ActiveOrHistoricCurrencyAndAmount",
    "AddressType2Code",
    "AllocationStatus1Choice",
    "AlternatePartyIdentification7",
    "AmountAndDirection44",
    "AmountAndDirection49",
    "BlockChainAddressWallet3",
    "CashAccountIdentification5Choice",
    "CashMovement7",
    "ClosingDate4Choice",
    "CollateralAmount14",
    "CollateralAmount5",
    "CollateralDate2",
    "CollateralParameters13",
    "CollateralParties8",
    "CollateralRole1Code",
    "CollateralStatus3Choice",
    "CollateralTransactionType1Choice",
    "CollateralTransactionType1Code",
    "CreditDebit3Code",
    "CreditDebitCode",
    "Date3Choice",
    "DateAndDateTime2Choice",
    "DateType2Code",
    "DealTransactionDetails7",
    "Document",
    "ExposureType14Code",
    "ExposureType23Choice",
    "FinancialInstrumentQuantity33Choice",
    "ForeignExchangeTerms23",
    "GenericIdentification30",
    "GenericIdentification36",
    "IdentificationSource3Choice",
    "IdentificationType42Choice",
    "NameAndAddress5",
    "OriginalAndCurrentQuantities1",
    "OtherIdentification1",
    "Pagination1",
    "PartyIdentification120Choice",
    "PartyIdentification136",
    "PartyIdentificationAndAccount193",
    "PartyIdentificationAndAccount202",
    "PartyIdentificationAndAccount203",
    "PostalAddress1",
    "ProprietaryReason4",
    "ProprietaryStatusAndReason6",
    "Quantity51Choice",
    "ReceiveDelivery1Code",
    "SecuritiesAccount19",
    "SecuritiesMovement8",
    "SecuritiesMovementStatus1Choice",
    "SecurityIdentification19",
    "SettlementStatus27Choice",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
    "TradingCapacity7Code",
    "TradingPartyCapacity5Choice",
    "TransactionIdentifications46",
    "TripartyCollateralStatusAdviceV01",
    "TypeOfIdentification1Code",
]
